#!/bin/sh

# Tested for:
# -----------
# - Fritz!Box 7312, Fritz!OS 06.30, 1&1 edition
# - Fritz!Box 7330 SL, Fritz!OS 06.30, 1&1 edition
# - Fritz!Box 7360, Fritz!OS 06.34, M-net edition
# - Fritz!Box 7430, Fritz!OS 06.83
# - Fritz!Box 7560 LI, Fritz!OS 06.92
# - (DOES NOT WORK on) Fritz!Box 6490 Cable, Fritz!OS 06.87, but on Fritz!OS 07.01
# - Fritz!Box 7490, Fritz!OS 07.01

CWD=$(dirname $0)
ROUTER_IP=$(/bin/sh $CWD/../general/get_router_ip.sh)

while getopts r:tduc opt
do
	case $opt in
		r) ROUTERIP=$OPTARG;;
		t) OPTION="test";;
		d) OPTION="download";;
		u) OPTION="upload";;
		c) OPTION="connection";;
	esac
done

create_xml(){
	xml_data="<?xml version='1.0' encoding='utf-8'?><s:Envelope s:encodingStyle='http://schemas.xmlsoap.org/soap/encoding/' xmlns:s='http://schemas.xmlsoap.org/soap/envelope/'><s:Body><u:$action xmlns:u='$uri'></u:$action></s:Body></s:Envelope>"
}

curl_request(){
	curl_output=$(curl -s -m 5 http://$ROUTERIP:49000$control_url -H "Content-Type: text/xml; charset='utf-8'" -H "SoapAction:$service#$action" -d "$xml_data")
}


invalidParameterException(){
	echo ""
	echo "Given parameter is invalid!"
	echo ""
}

dummy_curl_req(){
	service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
	action="GetAddonInfos"
	control_url="/igdupnp/control/WANCommonIFC1"
        create_xml
        curl_request
	test_output=$(echo $curl_output | grep NewByteSendRate)
}

show_total_bytes_sent(){
	service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
	action="GetTotalBytesSent"
	control_url="/igdupnp/control/WANCommonIFC1"
	create_xml
	curl_request
	totalBytesSent=$(echo $curl_output | awk -F"<NewTotalBytesSent>" '{print $2}' | awk -F"</NewTotalBytesSent>" '{print $1}')
}

show_total_bytes_received(){
        service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
        action="GetTotalBytesReceived"
        control_url="/igdupnp/control/WANCommonIFC1"
        create_xml
        curl_request
        totalBytesReceived=$(echo $curl_output | awk -F"<NewTotalBytesReceived>" '{print $2}' | awk -F"</NewTotalBytesReceived>" '{print $1}')
}

show_total_packets_received(){
        service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
        action="GetTotalPacketsReceived"
        control_url="/igdupnp/control/WANCommonIFC1"
        create_xml
        curl_request;
        totalPacketsReceived=$(echo $curl_output | awk -F"<NewTotalPacketsReceived>" '{print $2}' | awk -F"</NewTotalPacketsReceived>" '{print $1}')
}

show_total_packets_sent(){
        service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
        action="GetTotalPacketsSent"
        control_url="/igdupnp/control/WANCommonIFC1"
        create_xml
        curl_request
        totalPacketsSent=$(echo $curl_output | awk -F"<NewTotalPacketsSent>" '{print $2}' | awk -F"</NewTotalPacketsSent>" '{print $1}')
}

show_total_bytes_stats(){
	show_total_bytes_received
	show_total_bytes_sent
}

show_total_packets_stats(){
	show_total_packets_received
	show_total_packets_sent
}	

show_wan_stats(){
	show_total_packets_stats
	show_total_bytes_stats
}

connection_details(){
	service="urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1"
	action="GetCommonLinkProperties"
	control_url="/igdupnp/control/WANCommonIFC1"
	create_xml
	curl_request
	maxDownloadRate=$(echo $curl_output | awk -F"<NewLayer1DownstreamMaxBitRate>" '{print $2}' | awk -F"</NewLayer1DownstreamMaxBitRate>" '{print $1}')
	maxUploadRate=$(echo $curl_output | awk -F"<NewLayer1UpstreamMaxBitRate>" '{print $2}' | awk -F"</NewLayer1UpstreamMaxBitRate>" '{print $1}')
	echo "DSL Maximal Downloadrate: "$maxDownloadRate
	echo "DSL Maximal Uploadrate:   "$maxUploadRate
}

if [ -z "$ROUTERIP" ]; then
	echo "Router IP is missing:"
	echo "bash $CWD/fb_wan_stats.sh -r ROUTER_IP"
	exit 1
fi

dummy_curl_req

if [ $# -eq 2 ]
then 
	show_wan_stats
	connection_details
	echo "Total bytes reveived:     "$totalBytesReceived
	echo "Total bytes sent:         "$totalBytesSent
        echo "Total packets received:   "$totalPacketsReceived
        echo "Total packets sent:       "$totalPacketsSent
else
	case "$OPTION" in
		"upload")
			show_total_bytes_sent
			echo $totalBytesSent ;;
		"download")
			show_total_bytes_received
			echo $totalBytesReceived ;;
		"connection")
			connection_details ;;
		"test")
			echo $test_output ;;
		*)
			invalidParameterException ;;
	esac
fi
