#!/bin/bash

# get the necessary input from other parts of the system
CWD=$(dirname $0)

while getopts r:p:du opt
do
	case $opt in
		r) ROUTERIP=$OPTARG;;
		p) PASSWORD=$OPTARG;;
		d) OPTION="-d";;
		u) OPTION="-u";;
	esac
done

# these are specific files that need to be found
LOGIN_HTML="html/login/index.html"
LOGIN_JSON="data/Login.json"
LINKLAYER_JSON="engineer/data/linklayer.json"
DSL_JSON="engineer/data/dsl.json"

# exit status
IDLE=0
BUSY=1

# random number from login page
get_challenge(){
	CHALLENGE=$(curl -s -m 5 "http://$ROUTERIP/$LOGIN_HTML" | grep "challenge" | awk -F'"' '{print $2}')
}

# encrypted password
encrypt_sha256(){
	if [ -z "$PASSWORD" ]; then
		echo "Password is missing."
		exit 1
	fi
	PASSWORD_SHA256=$(echo -n "$CHALLENGE:$PASSWORD" | sha256sum | awk -F" " '{print $1}')
}

# router login
login(){
	login_out=$(curl -s -m 5 -c "$CWD/cookie.txt" "http://$ROUTERIP/$LOGIN_JSON" -H "Content-Type: application/x-www-form-urlencoded" -d "csrf_token=nulltoken&password=$PASSWORD_SHA256&showpw=0&challengev=$CHALLENGE")
        logged_in=$(echo $login_out | awk -F'"vartype":"status", "varid":"login", "varvalue":"' '{print $2}' | awk -F'"' '{print $1}')
        if [ "$logged_in" = "failed" ]
        then
                echo "Login failed!"
                exit 1
        fi

}

# link_layer page extraction
linklayer_page(){
	time1=$(date +%s%N | cut -b1-13)
	rand1=$(( RANDOM % 1001 ))
	time2=$(date +%s%N | cut -b1-13)
	rand2=$(( RANDOM % 1001 ))
	url="http://$ROUTERIP/$LINKLAYER_JSON?_time=$time1&_rand=$rand1&_time=$time2&_rand=$rand2"
	page_source=$(curl -s -m 5 -b "$CWD/cookie.txt" $url) 
	ppp256_in=$(echo $page_source | awk -F'"varid":"rx_packets", "varvalue":"' '{print $10}' | awk -F'"' '{print $1}')
        ppp256_out=$(echo $page_source | awk -F'"varid":"tx_packets", "varvalue":"' '{print $10}' | awk -F'"' '{print $1}')
}

# check if network is busy
calc_network_state(){
	linklayer_page
	out1=$ppp256_out
	in1=$ppp256_in
	through1=$(( out1+in1 ))
	# it seems the easiest implementation is to wait for exactly one second in between measurements. Other waiting times may need additional packages.
	sleep $SLEEPTIME
	linklayer_page
	out2=$ppp256_out
	in2=$ppp256_in
	through2=$(( out2+in2 ))
	through=$(( through2-through1 ))
	rm "$CWD/cookie.txt"
	if [ "$through" -gt "$THRESHOLD" ]
	then
		exit $BUSY
	else
        	exit $IDLE
	fi
}

print_results(){
        echo "Received frames:  $ppp256_in"
        echo "Sent frames:      $ppp256_out"
}


# fetch connection details
fetch_conn_details(){
	dsl_url="http://$ROUTERIP/$DSL_JSON"
	page_source=$(curl -s -m 5 -H "Content-Type: application/json, text/javascript, */*; Accept-Encoding: gzip, deflate" -b "$CWD/cookie.txt" $dsl_url)
	dl_rate=$(echo $page_source | jq '.[] | select(.varid == "ActualDataDown") | .varvalue' | tr -d '"')
	ul_rate=$(echo $page_source | jq '.[] | select(.varid == "ActualDataUp") | .varvalue' | tr -d '"')
	echo "Actual download rate: $dl_rate"
	echo "Actual upload rate:   $ul_rate"
        print_results
}

# procedure
get_challenge
encrypt_sha256
login
linklayer_page
if [ -z "$OPTION" ]; then
	fetch_conn_details
elif [ "$OPTION" = "-d" ]; then
	echo $ppp256_in
elif [ "$OPTION" = "-u" ]; then
	echo $ppp256_out
fi

rm "$CWD/cookie.txt"
